
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: AcceptQuest, C_GossipInfo, CompleteQuest, GetNumQuestChoices, GetQuestReward, IsQuestCompletable, next, ReadyCheckFrameYesButton

local IsShiftKeyDown = IsShiftKeyDown
local UnitIsDND = UnitIsDND

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Automation Busy")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M.GetCompletedQuests()
	return A.FilterTable(C_GossipInfo.GetActiveQuests(), function (quest) return quest.isComplete end)
end

function M:OnEnable()
	self:RegisterEvent("MODIFIER_STATE_CHANGED", "UpdateEvents")
	self:RegisterEvent("PLAYER_FLAGS_CHANGED", "UpdateEvents")
	self:UpdateEvents()
end

function M:UpdateEvents()
	if self:IsActive() then
		self:RegisterEvent("READY_CHECK")
		self:RegisterEvent("GOSSIP_SHOW")
		self:RegisterEvent("QUEST_GREETING")
		self:RegisterEvent("QUEST_DETAIL")
		self:RegisterEvent("QUEST_PROGRESS")
		self:RegisterEvent("QUEST_COMPLETE")
	else
		self:UnregisterEvent("READY_CHECK")
		self:UnregisterEvent("GOSSIP_SHOW")
		self:UnregisterEvent("QUEST_GREETING")
		self:UnregisterEvent("QUEST_DETAIL")
		self:UnregisterEvent("QUEST_PROGRESS")
		self:UnregisterEvent("QUEST_COMPLETE")
	end
end

-- Determine whether the module is active:
-- 1. The player is not DND and Shift is down.
-- 2. The player is DND and Shift is not down.
function M:IsActive()
	return (UnitIsDND("player") and not IsShiftKeyDown()) or (not UnitIsDND("player") and IsShiftKeyDown()) or false
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

-- Accept a ready check.
function M:READY_CHECK()
	ReadyCheckFrameYesButton:Click()
end

-- Select the first quest from a gossip list.
function M:GOSSIP_SHOW()
	local availableQuests = C_GossipInfo.GetAvailableQuests()
	local completedQuests = M.GetCompletedQuests()
	local gossipOptions = C_GossipInfo.GetOptions()

	if #availableQuests > 0 then
		C_GossipInfo.SelectAvailableQuest(availableQuests[1].questID)
	elseif #completedQuests > 0 then
		C_GossipInfo.SelectActiveQuest(completedQuests[1].questID)
	elseif #gossipOptions > 0 then
		if #gossipOptions == 1 then
			C_GossipInfo.SelectOption(gossipOptions[1].gossipOptionID)
		else
			for _, option in next, gossipOptions do
				if option.selectOptionWhenOnlyOption then
					C_GossipInfo.SelectOption(option.gossipOptionID)
					break
				end
			end
		end
	end
end

-- Select the first quest from a quest greeting.
function M:QUEST_GREETING()
	local availableQuests = C_GossipInfo.GetAvailableQuests()
	local completedQuests = M.GetCompletedQuests()

	if #availableQuests > 0 then
		C_GossipInfo.SelectAvailableQuest(availableQuests[1].questID)
	elseif #completedQuests > 0 then
		C_GossipInfo.SelectActiveQuest(completedQuests[1].questID)
	end
end

-- Accept an offered quest.
function M:QUEST_DETAIL()
	AcceptQuest()
end

-- Advance the quest completion dialog to the reward selection step.
function M:QUEST_PROGRESS()
	if IsQuestCompletable() then
		CompleteQuest()
	end
end

-- Complete the quest if there is only one reward.
function M:QUEST_COMPLETE()
	local numQuestChoices = GetNumQuestChoices()

	if numQuestChoices <= 1 then
		GetQuestReward(numQuestChoices)
	end
end
